@@ -2,7 +2,7 @@
 ####**********************************************************************
 ####
 ####  RANDOM FORESTS FOR SURVIVAL, REGRESSION, AND CLASSIFICATION (RF-SRC)
-####  Version 1.1.0
+####  Version 1.2
 ####
 ####  Copyright 2012, University of Miami
 ####
@@ -52,7 +52,7 @@
 ####    5425 Nestleway Drive, Suite L1
 ####    Clemmons, NC 27012
 ####
-####    email:  ubk@kogalur.com
+####    email:  commerce@kogalur.com
 ####    URL:    http://www.kogalur.com
 ####    --------------------------------------------------------------
 ####
@@ -64,7 +64,7 @@ var.select.rfsrc <-
   function(formula,          
            data,
            object,
-           cause,
+           cause, 
            method = c("md", "vh", "vh.vimp"),
            conservative = c("medium", "low", "high"),
            ntree = (if (method == "md") 1000 else 500),
@@ -86,32 +86,15 @@ var.select.rfsrc <-
            verbose = TRUE,
            ...
            )
-
 {
-
-  ###--------------------------------------------------------------
-  ###  
-  ###  workhorse: variable hunting algorithm
-  ###
-  ###--------------------------------------------------------------
-
-  
   rfsrc.var.hunting <- function(train.id, var.pt, nstep) {
-    ###------------------filtering step-----------------------
     if (verbose) cat("\t", paste("selecting variables using", mName), "...\n")
-
-    #which variables to include
     drop.var.pt <- setdiff(var.columns, var.pt)
-
-    #family specific checks
     if (grepl("surv", fmly)) {
       if (sum(data[train.id, 2]) < 2) {
         stop("training data has insufficient deaths: K is probably set too high\n")
       }
     }
-
-    # filtered forest
-    # over-ride user mtry setting: use an aggressive value
     rfsrc.filter.obj  <- rfsrc(rfsrc.all.f,
               data=(if (LENGTH(var.pt, drop.var.pt)) data[train.id, -drop.var.pt]
                     else data[train.id, ]),
@@ -123,16 +106,12 @@ var.select.rfsrc <-
               cause = cause,
               na.action = na.action,
               do.trace = do.trace)
-    ##set the target dimension for CR families
     if (rfsrc.filter.obj$family == "surv-CR") {
       target.dim <- max(1, min(cause, max(get.event.info(rfsrc.filter.obj)$event.type)), na.rm = TRUE)
     }
-
-    #extract vimp
+    rfsrc.filter.obj <- extract.imp.err(rfsrc.filter.obj, outcome.target)
     imp <- get.imp(rfsrc.filter.obj, target.dim)
     names(imp) <- rfsrc.filter.obj$xvar.names
-
-    #selection using vimp
     if (method == "vh.vimp") {
       VarStrength <- sort(imp, decreasing = TRUE)
       lower.VarStrength <- min(VarStrength) - 1 #need theoretical lower bound to vimp
@@ -140,12 +119,9 @@ var.select.rfsrc <-
       forest.depth <- m.depth <- NA
       sig.vars.old <- names(VarStrength)[1]
     }
-    
-    #selection using minimal depth
     else {
       max.obj <- max.subtree(rfsrc.filter.obj, conservative = (conservative == "high"))
       if (is.null(max.obj$order)) {
-        #maximal information failed; revert to vimp
         VarStrength <- lower.VarStrength <- 0
         forest.depth <- m.depth <- NA
         sig.vars.old <- names(sort(imp, decreasing = TRUE))[1]
@@ -163,12 +139,8 @@ var.select.rfsrc <-
         sig.vars.old <- names(VarStrength)[1]
       }
     }
-
-    #set nstep
     nstep <- max(round(length(rfsrc.filter.obj$xvar.names)/nstep), 1)
     imp.old <- 0
-  
-    ###regularized forward selection using joint vimp
     for (b in 1:nstep) {
       if (b == 1) {
         if (sum(VarStrength > lower.VarStrength) == 0) {
@@ -189,15 +161,12 @@ var.select.rfsrc <-
         sig.vars <- sig.vars.old
         break
       }
-      imp <- vimp(rfsrc.filter.obj, sig.vars, joint = TRUE)$importance[target.dim]
-
-      #verbose output
+      imp <- vimp(rfsrc.filter.obj, sig.vars, outcome.target = outcome.target,
+                  joint = TRUE)$importance[target.dim]
       if (verbose) cat("\t iteration: ", b,
                        "  # vars:",     length(sig.vars),
                        "  joint-vimp:",  round(imp, 3),
                        "\r")
-      
-      #break when joint vimp no longer increases (strict inequality is a safety feature)
       if (imp  <= imp.old) {
         sig.vars <- sig.vars.old
         break
@@ -207,12 +176,7 @@ var.select.rfsrc <-
         sig.vars.old <- sig.vars
         imp.old <- imp
       }
-      
     }
-    
-    ### refit forest and exit
-    ### over-ride user specified nodesize: use default mtry as variables 
-    ### should now be filtered and default settings should therefore apply
     var.pt <- var.columns[match(sig.vars, xvar.names)]
     drop.var.pt <- setdiff(var.columns, var.pt)
     rfsrc.obj  <- rfsrc(rfsrc.all.f,
@@ -224,18 +188,9 @@ var.select.rfsrc <-
                         cause = cause,
                         na.action = na.action,                  
                         do.trace = do.trace)
+    rfsrc.obj <- extract.imp.err(rfsrc.obj, outcome.target)
     return(list(rfsrc.obj=rfsrc.obj, sig.vars=rfsrc.obj$xvar.names, forest.depth=forest.depth, m.depth=m.depth))
-    
   }
-  
-  ###--------------------------------------------------------------
-  ###  
-  ###  preliminary processing
-  ###
-  ###--------------------------------------------------------------
-
-
-  ## Incoming parameter checks.  All are fatal.
   if (!missing(object)) {
     if (sum(inherits(object, c("rfsrc", "grow"), TRUE) == c(1, 2)) != 2) {
        stop("This function only works for objects of class `(rfsrc, grow)'")
@@ -246,7 +201,6 @@ var.select.rfsrc <-
   }
   else {
     if (missing(formula) || missing(data)) {
-      #allowance for users who overlook the correct way to assign the object
       if (sum(inherits(formula, c("rfsrc", "grow"), TRUE) == c(1, 2)) == 2) {
         object <- formula
       }
@@ -256,63 +210,41 @@ var.select.rfsrc <-
     }
     rfsrc.all.f <- formula
   }
-
-  ## assign key options
-  if (missing(cause)) cause <- 1
-  
-  ## Verify key options
-  method <- match.arg(method, c("md", "vh", "vh.vimp"))
-  conservative = match.arg(conservative, c("medium", "low", "high"))
-
-  ## rearrange the data 
   if (missing(object)) {
-
-    ## parse the formula
     formulaDetail <- finalizeFormula(parseFormula(rfsrc.all.f, data), data)
     fmly <- formulaDetail$family 
     xvar.names <- formulaDetail$xvar.names
     yvar.names <- formulaDetail$yvar.names
     data <- cbind(data[, yvar.names], data[, match(xvar.names, names(data))])
-    
   }
   else {
-
-    ## parse the object
     fmly <- object$family
     xvar.names <- object$xvar.names
     yvar.names <- object$yvar.names
     data <- data.frame(object$yvar, object$xvar)
-    
   }
   colnames(data) <- c(yvar.names, xvar.names)
   yvar <- data[, yvar.names]
-  
-  ## pretty names for method
+  if (missing(cause)) {
+    cause <- 1
+  }
+  outcome.target <- get.outcome.target(fmly, outcome.target)
+  method <- match.arg(method, c("md", "vh", "vh.vimp"))
+  conservative = match.arg(conservative, c("medium", "low", "high"))
   mName <- switch(method,
                   "md"      = "Minimal Depth",
                   "vh"      = "Variable Hunting",
                   "vh.vimp" = "Variable Hunting (VIMP)")
-
-  ## simplify the formula: needed when we drop variables
   rfsrc.all.f <- switch(fmly,
-       "surv"     =  as.formula(paste("Surv(",yvar.names[1],",",yvar.names[2],") ~ .")),
-       "surv-CR"  =  as.formula(paste("Surv(",yvar.names[1],",",yvar.names[2],") ~ .")),
-       "class"    =  as.formula(paste(yvar.names, "~ .")),
-       "regr"     =  as.formula(paste(yvar.names, "~ .")))
-    
-
-  ## initialize dimensions
+       "surv"   = as.formula(paste("Surv(",yvar.names[1],",",yvar.names[2],") ~ .")),
+       "surv-CR"= as.formula(paste("Surv(",yvar.names[1],",",yvar.names[2],") ~ .")),
+       "regr"   = as.formula(paste(yvar.names, "~ .")),
+       "class"  = as.formula(paste(yvar.names, "~ .")),
+  )
   n <- nrow(data)
   P <- length(xvar.names)
   r.dim <- ncol(cbind(yvar))
-
-  ## Specify the target event.  Later will be over-written for CR
   target.dim <- 1
-
-  ## Specify the default event type for CR
-  if (missing(cause)) cause <- 1
-
-  ## make special allowance for always.use x-variables
   var.columns <- (1 + r.dim):ncol(data)
   if (!is.null(always.use)) {
     always.use.pt <- var.columns[match(always.use, xvar.names)]
@@ -320,8 +252,6 @@ var.select.rfsrc <-
   else {
     always.use.pt <- NULL
   }
-
-  ## if xvar weight is specified, then make sure it is defined correctly
   if (!is.null(xvar.wt)) {
     if (any(xvar.wt < 0) | length(xvar.wt) != P | all(xvar.wt == 0)) {
       xvar.wt <- rep(1/P, P)
@@ -330,14 +260,10 @@ var.select.rfsrc <-
       xvar.wt <- xvar.wt / sum(xvar.wt)
     }
   }
-
-  
-  ## Final checks on option parameters  
   if (!is.null(mtry)) {
     mtry <- round(mtry)
     if (mtry < 1 | mtry > P) mtry <- max(1, min(mtry, P))
   }
-  ### prefit forest parameter details
   prefit.masterlist <- list(action = (method != "md"), ntree = 100, mtry = 500, nodesize = 3, nsplit = 1)
   parm.match <- na.omit(match(names(prefit), names(prefit.masterlist)))
   if (length(parm.match) > 0) {
@@ -347,60 +273,42 @@ var.select.rfsrc <-
   }
   prefit <- prefit.masterlist
   prefit.flag  <- prefit$action
-
-  ###--------------------------------------------------------------
-  ###  
-  ###  minimal depth analysis
-  ###
-  ###--------------------------------------------------------------
-
   if (method == "md") {
-
-    ###------------------------------------------------
-    ## run preliminary forest to determine weights for variables
-    ## we do this OUTSIDE of the loop
     if (prefit.flag && is.null(xvar.wt) && missing(object)) {
       if (verbose) cat("Using forests to select a variables likelihood of splitting a node...\n")
       rfsrc.prefit.obj  <- rfsrc(rfsrc.all.f,
-                                  data = data,
-                                  ntree = prefit$ntree,
-                                  nodesize = prefit$nodesize,
-                                  mtry = prefit$mtry,
-                                  splitrule = splitrule,
-                                  nsplit = prefit$nsplit,
-                                  cause = cause,
-                                  na.action = na.action)
-      ##set the target dimension for CR families
+                                 data = data,
+                                 ntree = prefit$ntree,
+                                 nodesize = prefit$nodesize,
+                                 mtry = prefit$mtry,
+                                 splitrule = splitrule,
+                                 nsplit = prefit$nsplit,
+                                 cause = cause,
+                                 na.action = na.action)
       if (rfsrc.prefit.obj$family == "surv-CR") {
         target.dim <- max(1, min(cause, max(get.event.info(rfsrc.prefit.obj)$event.type)), na.rm = TRUE)
       }
+      rfsrc.prefit.obj <- extract.imp.err(rfsrc.prefit.obj, outcome.target)
       wts <- pmax(get.imp(rfsrc.prefit.obj, target.dim), 0)
       if (any(wts > 0)) {
         xvar.wt <- wts
       }
     }
-
-    ###------------------------------------------------
-    ### extract minimal depth
-    
     if (!missing(object) && !prefit.flag) {
       if (verbose) cat("minimal depth variable selection ...\n")
       md.obj <- max.subtree(object, conservative = (conservative == "high"))
+      object <- extract.imp.err(object, outcome.target)
       pe <- get.err(object)
       ntree <- object$ntree
       nsplit <- object$nsplit
       mtry <- object$mtry
       nodesize <- object$nodesize
-      ##set the target dimension for CR families
       if (fmly == "surv-CR") {
         target.dim <- max(1, min(cause, max(get.event.info(object)$event.type)), na.rm = TRUE)
       }
       imp <- get.imp(object, target.dim)
       imp.all <- get.imp.all(object)
     }
-
-    ###------------------------------------------------
-    ### otherwise run forests...then extract minimal depth    
     else {
       if (verbose) cat("running forests ...\n")
       rfsrc.obj <- rfsrc(rfsrc.all.f,
@@ -414,12 +322,12 @@ var.select.rfsrc <-
                          na.action = na.action,
                          do.trace = do.trace,
                          xvar.wt = xvar.wt)
-      ##set the target dimension for CR families
       if (rfsrc.obj$family == "surv-CR") {
         target.dim <- max(1, min(cause, max(get.event.info(rfsrc.obj)$event.type)), na.rm = TRUE)
       }
       if (verbose) cat("minimal depth variable selection ...\n")
       md.obj <- max.subtree(rfsrc.obj, conservative = (conservative == "high"))
+      rfsrc.obj <- extract.imp.err(rfsrc.obj, outcome.target)
       pe <- get.err(rfsrc.obj)
       imp <- get.imp(rfsrc.obj, target.dim)
       imp.all <- get.imp.all(rfsrc.obj)
@@ -429,8 +337,6 @@ var.select.rfsrc <-
       fmly <- rfsrc.obj$family
       rm(rfsrc.obj)#don't need the grow object
     }
-
-    ### parse minimal depth information
     depth <- md.obj$order[, 1]
     threshold <- ifelse(conservative == "low", md.obj$threshold.1se, md.obj$threshold)
     top.var.pt <- (depth <= threshold)
@@ -439,9 +345,6 @@ var.select.rfsrc <-
     top.var.pt <- top.var.pt[o.r.m]
     varselect <- as.data.frame(cbind(depth = depth, vimp = imp.all))[o.r.m, ]
     topvars <- unique(c(always.use, rownames(varselect)[top.var.pt]))
-
-    ### fit a forest to final variable list
-    ### use default settings for nodesize, mtry due to dimension reduction
     if (refit == TRUE) {
       if (verbose) cat("fitting forests to minimal depth selected variables ...\n")
       var.pt <- var.columns[match(topvars, xvar.names)]
@@ -454,13 +357,11 @@ var.select.rfsrc <-
                nsplit = nsplit,
                na.action = na.action,
                do.trace = do.trace)
+      rfsrc.refit.obj <- extract.imp.err(rfsrc.refit.obj, outcome.target)
     }
     else {
       rfsrc.refit.obj <- NULL
     }
-
-
-    ### output: all nicely packaged
     if (verbose) {
       cat("\n\n")
       cat("-----------------------------------------------------------\n")
@@ -479,7 +380,6 @@ var.select.rfsrc <-
       cat("depth threshold    :", round(threshold, 4), "\n")
       if (!prefit.flag) {
         cat("PE (true OOB)      :", round(pe, 4), "\n")
-
       }
       else {
         cat("PE (biased)        :", round(pe, 4), "\n")
@@ -489,8 +389,6 @@ var.select.rfsrc <-
       print(round(varselect[top.var.pt, ], 3))
       cat("-----------------------------------------------------------\n")
     }
-   
-    ### Return the goodies
     return(invisible((list(err.rate=pe,
                            modelsize=modelsize,
                            topvars=topvars,
@@ -499,24 +397,9 @@ var.select.rfsrc <-
                            md.obj=md.obj
                            ))))
   }  
-
-  ###--------------------------------------------------------------
-  ###  
-  ###  VH algorithm
-  ###
-  ###--------------------------------------------------------------
- 
-
-  ### vectors/matrices etc.
   pred.results <- dim.results <- forest.depth <- rep(0, nrep)
   var.signature <- NULL
   var.depth <- matrix(NA, nrep, P)
-
-
-  ###------------------------------------------------
-  ## run preliminary forest to determine weights for variables
-  ## we do this OUTSIDE of the loop
-
   outside.loop <- FALSE
   if (prefit.flag & is.null(xvar.wt)) {
     if (verbose) cat("Using forests to select a variables likelihood of splitting a node...\n")
@@ -529,31 +412,19 @@ var.select.rfsrc <-
                                 cause = cause,
                                 splitrule = splitrule,
                                 na.action = na.action)
-    ##set the target dimension for CR families
     if (rfsrc.prefit.obj$family == "surv-CR") {
       target.dim <- max(1, min(cause, max(get.event.info(rfsrc.prefit.obj)$event.type)), na.rm = TRUE)
     }
-
-    ##record that a pre-fit has occurred
     outside.loop <- TRUE
   }
-
-  
-  ###------------------------------------------------
-  ## loop
-
   for (m in 1:nrep) {
-    
     if (verbose & nrep>1) cat("---------------------  Iteration:", m, "  ---------------------\n")
-  
-    ### train/test subsamples
-    ### use balanced sampling for CR/multiclass
     all.folds <- switch(fmly,
                         "surv"     =  balanced.folds(yvar[, 2], K),
                         "surv-CR"  =  balanced.folds(yvar[, 2], K),
                         "class"    =  balanced.folds(yvar, K),
-                        "regr"     =  cv.folds(n, K))
-    
+                        "regr"     =  cv.folds(n, K),
+                        )    
     if (fast == TRUE) {
       train.id <- all.folds[[1]]
       test.id <- all.folds[[2]]
@@ -562,9 +433,6 @@ var.select.rfsrc <-
       test.id <- all.folds[[1]]
       train.id <- setdiff(1:n, test.id)
     }
-
-    ## run preliminary forest to determine weights for variables
-    ## we do this INSIDE of the loop
     if (is.null(xvar.wt)) {
       if (!prefit.flag) {
         if (verbose) cat("Using forests to determine variable selection weights...\n")
@@ -577,11 +445,11 @@ var.select.rfsrc <-
                                     cause = cause,
                                     splitrule = splitrule,
                                     na.action = na.action)
-        ##set the target dimension for CR families
         if (rfsrc.prefit.obj$family == "surv-CR") {
           target.dim <- max(1, min(cause, max(get.event.info(rfsrc.prefit.obj)$event.type)), na.rm = TRUE)
         }
       }
+      rfsrc.prefit.obj <- extract.imp.err(rfsrc.prefit.obj, outcome.target)
       wts <- pmax(get.imp(rfsrc.prefit.obj, target.dim), 0)
       if (any(wts > 0)) {
         var.pt <- unique(resample(var.columns, mvars, replace = TRUE, prob = wts))
@@ -593,18 +461,12 @@ var.select.rfsrc <-
     else {
       var.pt <- var.columns[1:P]
     }
-
-    ### pre-guided gene selection
     if (!is.null(xvar.wt)) {
       var.pt <- unique(resample(var.columns, mvars, replace = TRUE, prob = xvar.wt))
     }
-   
-    ### always.use variables 
     if (!is.null(always.use)) {
       var.pt <- unique(c(var.pt, always.use.pt))
     }
-       
-    ### RFSRC gene hunting call
     object <- rfsrc.var.hunting(train.id, var.pt, nstep)
     rfsrc.obj <- object$rfsrc.obj
     sig.vars <- object$sig.vars
@@ -612,34 +474,20 @@ var.select.rfsrc <-
       forest.depth[m] <- object$forest.depth
       var.depth[m, match(names(object$m.depth), xvar.names)] <- object$m.depth
     }
-
-    #RFSRC prediction
     pred.out <- predict(rfsrc.obj,  data[test.id, ], importance = "none")
     pred.results[m] <- get.err(pred.out)[target.dim] 
     dim.results[m] <- length(sig.vars)
     var.signature <- c(var.signature, sig.vars)
-
-    #nice output
     if (verbose) {
       cat("\t                                                                \r")
       cat("\t PE:", round(pred.results[m], 4), "     dim:", dim.results[m], "\n")
     }
-
   }
-
-  ###--------------------------------------------------------------
-  ###  finalize details
-
-  ### remove NA's in PE 
   pred.results <- c(na.omit(pred.results))
-
-  ### frequency
   var.freq.all.temp <- 100 * tapply(var.signature, var.signature, length) / nrep
   freq.pt <- match(names(var.freq.all.temp), xvar.names)
   var.freq.all <- rep(0, P)
   var.freq.all[freq.pt] <- var.freq.all.temp
-
-  ###  package it up for output
   if (method == "vh") {
     var.depth.all <- apply(var.depth, 2, mean, na.rm = T)
     varselect <- cbind(depth = var.depth.all, rel.freq = var.freq.all)
@@ -652,8 +500,6 @@ var.select.rfsrc <-
   varselect <- varselect[o.r.f,, drop = FALSE]
   modelsize <- ceiling(mean(dim.results))  
   topvars <- unique(c(always.use, rownames(varselect)[1:modelsize]))
-  
-  ### fit a forest to final variable list
   if (refit == TRUE) {
     if (verbose) cat("fitting forests to final selected variables ...\n")
     var.pt <- var.columns[match(rownames(varselect)[1:modelsize], xvar.names)]
@@ -669,12 +515,9 @@ var.select.rfsrc <-
                         splitrule = splitrule,
                         do.trace = do.trace)
   }
-  
   else {
     rfsrc.refit.obj <- NULL
   }
- 
-  ### output: all nicely packaged
   if (verbose) {
     cat("\n\n")
     cat("-----------------------------------------------------------\n")
@@ -702,13 +545,10 @@ var.select.rfsrc <-
       cat("PE (K-fold)        :", round(mean(pred.results), 4), "+/-", round(SD(pred.results), 4), "\n")
     }
     cat("\n\n")
-    
     cat("Top variables:\n")
     print(round(varselect[1:modelsize,, drop = FALSE], 3))
     cat("-----------------------------------------------------------\n")
   }
-
-  ### return the goodies 
   return(invisible(list(err.rate=pred.results,
                         modelsize=modelsize,
                         topvars=topvars,
@@ -716,26 +556,18 @@ var.select.rfsrc <-
                         rfsrc.refit.obj=rfsrc.refit.obj,
                         md.obj=NULL
                         )))
-            
-
-
 }
-
-###--------------------------------------------------------------
-###  
-###  useful functions
-###
-###--------------------------------------------------------------
-
+  extract.imp.err <- function(f.o, outcome.target) {
+    f.o
+  }
   get.imp <- function(f.o, target.dim) {
     if (!is.null(f.o$importance)) {
       c(cbind(f.o$importance)[, target.dim])
-    }
+      }
     else {
       rep(NA, length(f.o$xvar.names))
     }
   }
-
   get.imp.all <- function(f.o) {
     if (!is.null(f.o$importance)) {
       imp.all <- cbind(f.o$importance)
@@ -751,7 +583,6 @@ var.select.rfsrc <-
       rep(NA, length(f.o$xvar.names))
     }
   }
-
   get.err <- function(f.o) {
     if (!is.null(f.o$err.rate)) {
       if (grepl("surv", f.o$family)) {
@@ -765,7 +596,6 @@ var.select.rfsrc <-
       NA
     }
   }
-
   SD <- function(x) {
     if (all(is.na(x))) {
       NA
@@ -774,11 +604,9 @@ var.select.rfsrc <-
       sd(x, na.rm = TRUE)
     }
   }
-
   LENGTH <- function(x, y) {
     (length(x) > 0 & length(y) > 0)
   }
-
   Mtry <- function(x, y) {
     mtry <- round((length(x) - length(y))/3)
     if (mtry == 0) {
@@ -788,19 +616,15 @@ var.select.rfsrc <-
       mtry
     }
   }
-  
-
   cv.folds <- function (n, folds = 10) {
     split(resample(1:n), rep(1:folds, length = n))
   }
-
   permute.rows <-function(x) {
     n <- nrow(x)
     p <- ncol(x)
     mm <- runif(length(x)) + rep(seq(n) * 10, rep(p, n))
     matrix(t(x)[order(mm)], n, p, byrow = TRUE)
   }
-
   balanced.folds <- function(y, nfolds = min(min(table(y)), 10)) {
     y[is.na(y)] <- resample(y[!is.na(y)], size = sum(is.na(y)), replace = TRUE)
     totals <- table(y)
@@ -828,6 +652,4 @@ var.select.rfsrc <-
       return(res)
     }
   }
-
-
 var.select <- var.select.rfsrc
