@@ -2,7 +2,7 @@
 ////**********************************************************************
 ////
 ////  RANDOM FORESTS FOR SURVIVAL, REGRESSION, AND CLASSIFICATION (RF-SRC)
-////  Version 1.1.0
+////  Version 1.2
 ////
 ////  Copyright 2012, University of Miami
 ////
@@ -52,7 +52,7 @@
 ////    5425 Nestleway Drive, Suite L1
 ////    Clemmons, NC 27012
 ////
-////    email:  ubk@kogalur.com
+////    email:  commerce@kogalur.com
 ////    URL:    http://www.kogalur.com
 ////    --------------------------------------------------------------
 ////
@@ -67,13 +67,16 @@
 #include        "random.h"
 #include       "nodeOps.h"
 #include     "bootstrap.h"
+#include    "splitUtil.h"
 #include      "treeUtil.h"
 #include     "factorOps.h"
+#include    "regression.h"
 #include        "impute.h"
 char imputeNode (uint     type,
+                 char     lmvFlag,
                  char     chainFlag,
                  uint     treeID, 
-                 Node    *parent,
+                 Node    *nodePtr,
                  uint    *repMembrIndx,
                  uint     repMembrSize,
                  uint    *allMembrIndx,
@@ -81,27 +84,47 @@ char imputeNode (uint     type,
   double  **response;
   double  **predictor;
   uint    *mRecordMap;
-  uint     mvSize;
+  uint     mvSignSize;
   int    **mvSign;
   int     *mvIndex;
   int     *mvNSptr;
+  uint     mRecordSize;
   double *valuePtr, *imputePtr;
   char mPredictorFlag;
   uint unsignedIndexSource, unsignedIndexTarget;
   char result;
+  uint  *glmvIndexPtr;
+  uint  *glmvIndexSize;
+  uint  *glmvIndexParentPtr;
+  uint   glmvIndexParentSize;
+  uint  *glmrIndexPtr;
+  uint  *glmrIndexSize;
+  uint  *glmrIndexParentPtr;
+  uint   glmrIndexParentSize;
+  char mvFlag;
   uint i,p;
   uint localDistributionSize;
-  mvNSptr = 0;  
-  mvIndex = 0;  
-  mvSign  = 0;  
-  mvSize  = 0;  
+  mvNSptr = NULL;  
+  mvIndex = NULL;  
+  mvSign  = NULL;  
+  mvSignSize  = 0;  
   mRecordMap = NULL;  
+  mRecordSize = 0;    
   predictor  = NULL;  
   response   = NULL;  
+  glmvIndexPtr = NULL;
+  glmvIndexSize = NULL;
+  glmvIndexParentPtr = NULL;
+  glmvIndexParentSize = 0;
+  glmrIndexPtr = NULL;
+  glmrIndexSize = NULL;
+  glmrIndexParentPtr = NULL;
+  glmrIndexParentSize = 0;
   result = FALSE;
   switch (type) {
   case RF_PRED:
-    if (RF_fmRecordSize > 0) {
+    mRecordSize = RF_fmRecordSize;
+    if (mRecordSize > 0) {
       if (RF_frSize > 0) {
         response = RF_fresponse[treeID];
       } 
@@ -110,22 +133,147 @@ char imputeNode (uint     type,
       }
       predictor = RF_fobservation[treeID];
       mRecordMap = RF_fmRecordMap;
-      mvSize = RF_fmvSize;
+      mvSignSize = RF_fmvSignSize;
       mvSign = RF_fmvSign;
       mvIndex = RF_fmvIndex;
-      mvNSptr = parent -> fmvSign;
+      mvNSptr = nodePtr -> fmvSign;
+      if (lmvFlag) {
+        if((nodePtr -> parent) == NULL) {
+          glmvIndexParentPtr = uivector(1, mvSignSize);
+          glmvIndexParentSize = mvSignSize;
+          for (p = 1; p <= glmvIndexParentSize; p++) {
+            glmvIndexParentPtr[p] = p;
+          }
+          glmrIndexParentPtr = uivector(1, mRecordSize);
+          glmrIndexParentSize = mRecordSize;
+          for (i = 1; i <= glmrIndexParentSize; i++) {
+            glmrIndexParentPtr[i] = i;
+          }
+          stackNodeFLMVIndex(nodePtr, glmvIndexParentSize);
+          glmvIndexPtr  = nodePtr -> flmvIndex;
+          glmvIndexSize = & (nodePtr -> flmvIndexActualSize);
+          *glmvIndexSize = 0;
+          stackNodeFLMRIndex(nodePtr, glmrIndexParentSize);
+          glmrIndexPtr  = nodePtr -> flmrIndex;
+          glmrIndexSize = & (nodePtr -> flmrIndexActualSize);
+          *glmrIndexSize = 0;
+        }
+        else {
+          if((nodePtr -> parent) -> flmvIndexActualSize > 0) {
+            glmvIndexParentPtr = (nodePtr -> parent) -> flmvIndex;
+            glmvIndexParentSize = (nodePtr -> parent) -> flmvIndexActualSize;
+            stackNodeFLMVIndex(nodePtr, glmvIndexParentSize);
+            glmvIndexPtr  = nodePtr -> flmvIndex;
+            glmvIndexSize = & (nodePtr -> flmvIndexActualSize);
+            *glmvIndexSize = 0;
+          }
+          else {
+            glmvIndexParentPtr  = NULL;
+            glmvIndexParentSize = 0;
+            glmvIndexPtr = glmvIndexSize = NULL;
+          }
+          if((nodePtr -> parent) -> flmrIndexActualSize > 0) {
+            glmrIndexParentPtr = (nodePtr -> parent) -> flmrIndex;
+            glmrIndexParentSize = (nodePtr -> parent) -> flmrIndexActualSize;
+            stackNodeFLMRIndex(nodePtr, glmrIndexParentSize);
+            glmrIndexPtr  = nodePtr -> flmrIndex;
+            glmrIndexSize = & (nodePtr -> flmrIndexActualSize);
+            *glmrIndexSize = 0;
+          }
+          else {
+            glmrIndexParentPtr = NULL;
+            glmrIndexParentSize = 0;
+            glmrIndexPtr = glmrIndexSize = NULL;
+          }
+        }
+      }  
+      else {
+        glmvIndexParentPtr = uivector(1, mvSignSize);
+        glmvIndexParentSize = mvSignSize;
+        for (p = 1; p <= glmvIndexParentSize; p++) {
+          glmvIndexParentPtr[p] = p;
+        }
+        glmrIndexParentPtr = uivector(1, mRecordSize);
+        glmrIndexParentSize = mRecordSize;
+        for (i = 1; i <= glmrIndexParentSize; i++) {
+          glmrIndexParentPtr[i] = i;
+        }
+      }
       result = TRUE;
     }
     break;
   default:
-    if (RF_mRecordSize > 0) {
+    mRecordSize = RF_mRecordSize;
+    if (mRecordSize > 0) {
       response = RF_response[treeID];
       predictor = RF_observation[treeID];
       mRecordMap = RF_mRecordMap;
-      mvSize = RF_mvSize;
+      mvSignSize = RF_mvSignSize;
       mvSign = RF_mvSign;
       mvIndex = RF_mvIndex;
-      mvNSptr = parent -> mvSign;
+      mvNSptr = nodePtr -> mvSign;
+      if (lmvFlag) {
+        if((nodePtr -> parent) == NULL) {
+          glmvIndexParentPtr = uivector(1, mvSignSize);
+          glmvIndexParentSize = mvSignSize;
+          for (p = 1; p <= glmvIndexParentSize; p++) {
+            glmvIndexParentPtr[p] = p;
+          }
+          glmrIndexParentPtr = uivector(1, mRecordSize);
+          glmrIndexParentSize = mRecordSize;
+          for (i = 1; i <= glmrIndexParentSize; i++) {
+            glmrIndexParentPtr[i] = i;
+          }
+          stackNodeLMVIndex(nodePtr, glmvIndexParentSize);
+          glmvIndexPtr  = nodePtr -> lmvIndex;
+          glmvIndexSize = & (nodePtr -> lmvIndexActualSize);
+          *glmvIndexSize = 0;
+          stackNodeLMRIndex(nodePtr, glmrIndexParentSize);
+          glmrIndexPtr  = nodePtr -> lmrIndex;
+          glmrIndexSize = & (nodePtr -> lmrIndexActualSize);
+          *glmrIndexSize = 0;
+        }
+        else {
+          if((nodePtr -> parent) -> lmvIndexActualSize > 0) {
+            glmvIndexParentPtr = (nodePtr -> parent) -> lmvIndex;
+            glmvIndexParentSize = (nodePtr -> parent) -> lmvIndexActualSize;
+            stackNodeLMVIndex(nodePtr, glmvIndexParentSize);
+            glmvIndexPtr  = nodePtr -> lmvIndex;
+            glmvIndexSize = & (nodePtr -> lmvIndexActualSize);
+            *glmvIndexSize = 0;
+          }
+          else {
+            glmvIndexParentPtr = NULL;
+            glmvIndexParentSize = 0;
+            glmvIndexPtr = glmvIndexSize = NULL;
+          }
+          if((nodePtr -> parent) -> lmrIndexActualSize > 0) {
+            glmrIndexParentPtr = (nodePtr -> parent) -> lmrIndex;
+            glmrIndexParentSize = (nodePtr -> parent) -> lmrIndexActualSize;
+            stackNodeLMRIndex(nodePtr, glmrIndexParentSize);
+            glmrIndexPtr  = nodePtr -> lmrIndex;
+            glmrIndexSize = & (nodePtr -> lmrIndexActualSize);
+            *glmrIndexSize = 0;
+          }
+          else {
+            glmrIndexParentPtr = NULL;
+            glmrIndexParentSize = 0;
+            glmrIndexPtr = glmrIndexSize = NULL;
+          }
+        }
+      }  
+      else {
+        glmvIndexParentPtr = uivector(1, mvSignSize);
+        glmvIndexParentSize = mvSignSize;
+        for (p = 1; p <= glmvIndexParentSize; p++) {
+          glmvIndexParentPtr[p] = p;
+        }
+        glmrIndexParentPtr = uivector(1, mRecordSize);
+        glmrIndexParentSize = mRecordSize;
+        for (i = 1; i <= glmrIndexParentSize; i++) {
+          glmrIndexParentPtr[i] = i;
+        }
+      }
       result = TRUE;
     }
     break;
@@ -137,57 +285,81 @@ char imputeNode (uint     type,
     error("\nRF-SRC:  The application will now exit.\n");
   }
   double *localDistribution = dvector(1, repMembrSize + 1);
-  for (p = 1; p <= mvSize; p++) {
-    if (mvNSptr[p] != -1) {
-      if (mvIndex[p] < 0) {
-        unsignedIndexSource = unsignedIndexTarget = (uint) abs(mvIndex[p]);
-        valuePtr = RF_response[treeID][(uint) abs(mvIndex[p])];
-        imputePtr = response[(uint) abs(mvIndex[p])];
+  for (p = 1; p <= glmvIndexParentSize; p++) {
+    if (mvNSptr[glmvIndexParentPtr[p]] != -1) {
+      if (mvIndex[glmvIndexParentPtr[p]] < 0) {
+        unsignedIndexSource = unsignedIndexTarget = (uint) abs(mvIndex[glmvIndexParentPtr[p]]);
+        valuePtr = RF_response[treeID][(uint) abs(mvIndex[glmvIndexParentPtr[p]])];
+        imputePtr = response[(uint) abs(mvIndex[glmvIndexParentPtr[p]])];
       }
       else { 
-        unsignedIndexSource = RF_rSize + (uint) mvIndex[p];
+        unsignedIndexSource = RF_rSize + (uint) mvIndex[glmvIndexParentPtr[p]];
         if (type == RF_PRED) {
           if (RF_frSize > 0) {
-            unsignedIndexTarget = RF_rSize + (uint) mvIndex[p];
+            unsignedIndexTarget = RF_rSize + (uint) mvIndex[glmvIndexParentPtr[p]];
           }
           else {
-            unsignedIndexTarget = (uint) mvIndex[p];
+            unsignedIndexTarget = (uint) mvIndex[glmvIndexParentPtr[p]];
           }
         }
         else {
-          unsignedIndexTarget = RF_rSize + (uint) mvIndex[p];
+          unsignedIndexTarget = RF_rSize + (uint) mvIndex[glmvIndexParentPtr[p]];
         }
-        valuePtr = RF_observation[treeID][(uint) mvIndex[p]];
-        imputePtr = predictor[(uint) mvIndex[p]];
+        valuePtr = RF_observation[treeID][(uint) mvIndex[glmvIndexParentPtr[p]]];
+        imputePtr = predictor[(uint) mvIndex[glmvIndexParentPtr[p]]];
       }
       localDistributionSize = 0;
       for (i = 1; i <= repMembrSize; i++) {
-          mPredictorFlag = TRUE;
-          if (RF_mRecordMap[repMembrIndx[i]] == 0) {
-            mPredictorFlag = FALSE;
-          }
-          else if (RF_mvSign[unsignedIndexSource][RF_mRecordMap[repMembrIndx[i]]] == 0) {
-            mPredictorFlag = FALSE;
-          }
-          if (mPredictorFlag == FALSE) {
-            localDistributionSize ++;
-            localDistribution[localDistributionSize] = valuePtr[repMembrIndx[i]];
-          }
+        mPredictorFlag = TRUE;
+        if (RF_mRecordMap[repMembrIndx[i]] == 0) {
+          mPredictorFlag = FALSE;
+        }
+        else if (RF_mvSign[unsignedIndexSource][RF_mRecordMap[repMembrIndx[i]]] == 0) {
+          mPredictorFlag = FALSE;
+        }
+        if (mPredictorFlag == FALSE) {
+          localDistributionSize ++;
+          localDistribution[localDistributionSize] = valuePtr[repMembrIndx[i]];
+        }
       }  
-      if (localDistributionSize > 0) {
-        for (i = 1; i <= allMembrSize; i++) {
-          if (mRecordMap[allMembrIndx[i]] > 0) {
-            if(mvSign[unsignedIndexTarget][mRecordMap[allMembrIndx[i]]] == 1) {
+      mvFlag = FALSE;
+      for (i = 1; i <= allMembrSize; i++) {
+        if (mRecordMap[allMembrIndx[i]] > 0) {
+          if(mvSign[unsignedIndexTarget][mRecordMap[allMembrIndx[i]]] == 1) {
+            mvFlag = TRUE;
+            if (localDistributionSize > 0) {
               imputePtr[allMembrIndx[i]] = getSampleValue(localDistribution, localDistributionSize, chainFlag, treeID);
-            }  
-          }
-        }  
+            }
+          }  
+        }
       }  
-      else {
+      if (lmvFlag) {
+        if (mvFlag) {
+          glmvIndexPtr[++(*glmvIndexSize)] = glmvIndexParentPtr[p]; 
+        }
+      }
+      if (localDistributionSize == 0) {
       }
     }  
   }  
+  if (lmvFlag) {
+    for (i = 1; i <= allMembrSize; i++) {
+      if (mRecordMap[allMembrIndx[i]] > 0) {
+        glmrIndexPtr[++(*glmrIndexSize)] = mRecordMap[allMembrIndx[i]];
+      }
+    }
+  }
   free_dvector(localDistribution, 1, repMembrSize + 1);
+  if (lmvFlag) {
+    if((nodePtr -> parent) == NULL) {
+      free_uivector(glmvIndexParentPtr, 1, mvSignSize);
+      free_uivector(glmrIndexParentPtr, 1, mRecordSize);
+    }
+  }
+  else {
+    free_uivector(glmvIndexParentPtr, 1, mvSignSize);
+    free_uivector(glmrIndexParentPtr, 1, mRecordSize);
+  }
   return TRUE;
 }
 char restoreNodeMembership(uint  mode, 
@@ -248,6 +420,8 @@ char restoreNodeMembership(uint  mode,
         }
         bsUpdateFlag = TRUE;
       }
+      repMembrIndx = bootMembrIndx;
+      repMembrSize = bootMembrSize;
     }
   }
   else {
@@ -259,6 +433,7 @@ char restoreNodeMembership(uint  mode,
   if (bootResult) {
     if (RF_mRecordSize > 0) {
       imputeNode(RF_GROW,
+                 TRUE,
                  TRUE,
                  treeID,
                  parent,
@@ -283,6 +458,7 @@ char restoreNodeMembership(uint  mode,
     case RF_PRED:
       if (RF_fmRecordSize > 0) {
         imputeNode(RF_PRED,
+                   TRUE,
                    FALSE,
                    treeID,
                    parent,
@@ -303,6 +479,9 @@ char restoreNodeMembership(uint  mode,
     }
   }  
   if (bootResult) {
+    if ((RF_opt & OPT_NODE_STAT) || (RF_ptnCount > 0)) {
+      getVariance(repMembrSize, repMembrIndx, RF_response[treeID][RF_rTarget], NULL, & (parent -> variance));  
+    }
     if (((parent -> left) != NULL) && ((parent -> right) != NULL)) {
       tnUpdateFlag = FALSE;
       uint *membershipIndicator = uivector(1, RF_observationSize);
@@ -320,12 +499,12 @@ char restoreNodeMembership(uint  mode,
         if (daughterFlag == LEFT) {
           membershipIndicator[allMembrIndx[i]] = LEFT;
           leftAllMembrSize ++;
-          RF_nodeMembership[treeID][allMembrIndx[i]] = parent -> left;
+          RF_tNodeMembership[treeID][allMembrIndx[i]] = parent -> left;
         }
         else {
           membershipIndicator[allMembrIndx[i]] = RIGHT;
           rghtAllMembrSize ++;
-          RF_nodeMembership[treeID][allMembrIndx[i]] = parent -> right;
+          RF_tNodeMembership[treeID][allMembrIndx[i]] = parent -> right;
         }
       }
       leftAllMembrIndx  = uivector(1, leftAllMembrSize + 1);
@@ -375,12 +554,12 @@ char restoreNodeMembership(uint  mode,
           if (daughterFlag == LEFT) {
             ngMembershipIndicator[ngAllMembrIndx[i]] = LEFT;
             ngLeftAllMembrSize ++;
-            RF_fnodeMembership[treeID][ngAllMembrIndx[i]] = parent -> left;
+            RF_ftNodeMembership[treeID][ngAllMembrIndx[i]] = parent -> left;
           }
           else {
             ngMembershipIndicator[ngAllMembrIndx[i]] = RIGHT;
             ngRghtAllMembrSize ++;
-            RF_fnodeMembership[treeID][ngAllMembrIndx[i]] = parent -> right;
+            RF_ftNodeMembership[treeID][ngAllMembrIndx[i]] = parent -> right;
           }
         }
         ngLeftAllMembrIndx  = uivector(1, ngLeftAllMembrSize + 1);
@@ -469,7 +648,7 @@ void imputeUpdateSummary (uint     mode,
                           uint     treeID) {
   uint     mRecordSize;
   uint    *mRecordIndex;
-  uint     mvSize;
+  uint     mvSignSize;
   int    **mvSign;
   int     *mvIndex;
   int     *mvNodeSign;
@@ -481,7 +660,7 @@ void imputeUpdateSummary (uint     mode,
   rspSize = 0;  
   mvIndex = 0;  
   mvSign  = 0;  
-  mvSize  = 0;  
+  mvSignSize  = 0;  
   mRecordIndex = 0;  
   mRecordSize  = 0;  
   result = FALSE;
@@ -489,7 +668,7 @@ void imputeUpdateSummary (uint     mode,
     if (RF_mRecordSize > 0) {
       mRecordSize = RF_mRecordSize;
       mRecordIndex = RF_mRecordIndex;
-      mvSize = RF_mvSize;
+      mvSignSize = RF_mvSignSize;
       mvSign = RF_mvSign;
       mvIndex = RF_mvIndex;
       rspSize = RF_rSize;
@@ -500,7 +679,7 @@ void imputeUpdateSummary (uint     mode,
     if (RF_fmRecordSize > 0) {
       mRecordSize = RF_fmRecordSize;
       mRecordIndex = RF_fmRecordIndex;
-      mvSize = RF_fmvSize;
+      mvSignSize = RF_fmvSignSize;
       mvSign = RF_fmvSign;
       mvIndex = RF_fmvIndex;
       rspSize = RF_frSize;
@@ -513,13 +692,13 @@ void imputeUpdateSummary (uint     mode,
     Rprintf("\nRF-SRC:  Please Contact Technical Support.");
     error("\nRF-SRC:  The application will now exit.\n");
   }
-  for (p = 1; p <= mvSize; p++) {
+  for (p = 1; p <= mvSignSize; p++) {
     for (i = 1; i <= mRecordSize; i++) {
       if ((mode == RF_GROW) || (mode == RF_REST)) {
-          mvNodeSign = RF_nodeMembership[treeID][mRecordIndex[i]] -> mvSign;
+          mvNodeSign = RF_tNodeMembership[treeID][mRecordIndex[i]] -> mvSign;
       }
       else {
-          mvNodeSign = RF_fnodeMembership[treeID][mRecordIndex[i]] -> fmvSign;
+          mvNodeSign = RF_ftNodeMembership[treeID][mRecordIndex[i]] -> fmvSign;
       }
       if (mvNodeSign[p] != -1) {
         if (mvIndex[p] < 0) {
@@ -535,116 +714,13 @@ void imputeUpdateSummary (uint     mode,
             Rprintf("\nDiagnostic Trace of Shadowed Data:  ");
             Rprintf("\n       index   imputation -> \n");
             Rprintf(  "            ");
-            for (p=1; p <= mvSize; p++) {
+            for (p=1; p <= mvSignSize; p++) {
               Rprintf(" %12d", mvIndex[p]);
             }
             Rprintf("\n");
             for (i = 1; i <= mRecordSize; i++) {
               Rprintf("%12d", mRecordIndex[i]);
-              for (p = 1; p <= mvSize; p++) {
-                if (mvIndex[p] < 0) {
-                  valuePtr = responsePtr[(uint) abs(mvIndex[p])];
-                }
-                else {
-                  valuePtr = predictorPtr[(uint) mvIndex[p]];
-                }
-                Rprintf(" %12.4f", valuePtr[mRecordIndex[i]]);
-              }
-              Rprintf("\n");
-            }
-            Rprintf("\nRF-SRC:  *** ERROR *** ");
-            Rprintf("\nRF-SRC:  Attempt to update forest impute data with invalid shadowed value, NA. ");
-            Rprintf("\nRF-SRC:  Invalid value for:  [indv][outcome/predictor] = [%10d][%10d] ", mRecordIndex[i], mvIndex[p]);
-            Rprintf("\nRF-SRC:  Please Contact Technical Support.");
-            error("\nRF-SRC:  The application will now exit.\n");
-          }  
-          else {
-            RF_dmvImputation[treeID][i][p] = valuePtr[mRecordIndex[i]];
-          }
-        }  
-      }  
-    }  
-  }  
-}
-void imputeUpdateSummaryNew (uint     mode, 
-                          double **responsePtr, 
-                          double **predictorPtr, 
-                          uint     treeID) {
-  uint     mRecordSize;
-  uint    *mRecordIndex;
-  uint     mvSize;
-  int    **mvSign;
-  int     *mvIndex;
-  int     *mvNodeSign;
-  double  *valuePtr;
-  uint     unsignedIndex;
-  char result;
-  uint rspSize;
-  uint i, p;
-  rspSize = 0;  
-  mvIndex = 0;  
-  mvSign  = 0;  
-  mvSize  = 0;  
-  mRecordIndex = 0;  
-  mRecordSize  = 0;  
-  result = FALSE;
-  if ((mode == RF_GROW) || (mode == RF_REST)) {
-    if (RF_mRecordSize > 0) {
-      mRecordSize = RF_mRecordSize;
-      mRecordIndex = RF_mRecordIndex;
-      mvSize = RF_mvSize;
-      mvSign = RF_mvSign;
-      mvIndex = RF_mvIndex;
-      rspSize = RF_rSize;
-      result = TRUE;
-    }
-  }
-  else {
-    if (RF_fmRecordSize > 0) {
-      mRecordSize = RF_fmRecordSize;
-      mRecordIndex = RF_fmRecordIndex;
-      mvSize = RF_fmvSize;
-      mvSign = RF_fmvSign;
-      mvIndex = RF_fmvIndex;
-      rspSize = RF_frSize;
-      result = TRUE;
-    }
-  }
-  if (result == FALSE) {
-    Rprintf("\nRF-SRC:  *** ERROR *** ");
-    Rprintf("\nRF-SRC:  Attempt to update forest impute data with no missingness in mode:  %10d", mode);
-    Rprintf("\nRF-SRC:  Please Contact Technical Support.");
-    error("\nRF-SRC:  The application will now exit.\n");
-  }
-  for (p = 1; p <= mvSize; p++) {
-    for (i = 1; i <= mRecordSize; i++) {
-      if ((mode == RF_GROW) || (mode == RF_REST)) {
-          mvNodeSign = RF_nodeMembership[treeID][mRecordIndex[i]] -> mvSign;
-      }
-      else {
-          mvNodeSign = RF_fnodeMembership[treeID][mRecordIndex[i]] -> fmvSign;
-      }
-      if (mvNodeSign[p] != -1) {
-        if (mvIndex[p] < 0) {
-          unsignedIndex = (uint) abs(mvIndex[p]);
-          valuePtr = responsePtr[(uint) abs(mvIndex[p])];
-        }
-        else {
-          unsignedIndex = (uint) mvIndex[p] + rspSize;
-          valuePtr = predictorPtr[(uint) mvIndex[p]];
-        }
-        if (mvSign[unsignedIndex][i] == 1) {
-          if (ISNA(valuePtr[mRecordIndex[i]])) {
-            Rprintf("\nDiagnostic Trace of Shadowed Data:  ");
-            Rprintf("\n       index   imputation -> \n");
-            Rprintf(  "            ");
-            for (p=1; p <= mvSize; p++) {
-              Rprintf(" %12d", mvIndex[p]);
-            }
-            Rprintf("\n");
-            for (i = 1; i <= mRecordSize; i++) {
-              Rprintf("%12d", mRecordIndex[i]);
-              for (p = 1; p <= mvSize; p++) {
+              for (p = 1; p <= mvSignSize; p++) {
                 if (mvIndex[p] < 0) {
                   valuePtr = responsePtr[(uint) abs(mvIndex[p])];
                 }
@@ -675,7 +751,7 @@ void imputeUpdateShadow (uint      mode,
                          double  **shadowPredictor) {
   uint     mRecordSize;
   uint    *mRecordIndex;
-  uint     mvSize;
+  uint     mvSignSize;
   int    **mvSign;
   int     *mvIndex;
   double **outResponse;
@@ -688,7 +764,7 @@ void imputeUpdateShadow (uint      mode,
   uint i, p;
   mRecordSize  = 0;     
   mRecordIndex = NULL;  
-  mvSize       = 0;     
+  mvSignSize       = 0;     
   mvSign       = NULL;  
   mvIndex      = NULL;  
   outResponse  = NULL;  
@@ -700,7 +776,7 @@ void imputeUpdateShadow (uint      mode,
   case RF_PRED:
     mRecordSize = RF_fmRecordSize;
     mRecordIndex = RF_fmRecordIndex;
-    mvSize = RF_fmvSize;
+    mvSignSize = RF_fmvSignSize;
     mvSign = RF_fmvSign;
     mvIndex = RF_fmvIndex;
     outResponse  = RF_sImputeResponsePtr;
@@ -710,7 +786,7 @@ void imputeUpdateShadow (uint      mode,
   default:
     mRecordSize = RF_mRecordSize;
     mRecordIndex = RF_mRecordIndex;
-    mvSize = RF_mvSize;
+    mvSignSize = RF_mvSignSize;
     mvSign = RF_mvSign;
     mvIndex = RF_mvIndex;
     if ((selectionFlag == TRUE) || (selectionFlag == ACTIVE)) {
@@ -731,7 +807,7 @@ void imputeUpdateShadow (uint      mode,
     error("\nRF-SRC:  The application will now exit.\n");
   }
   outcomeFlag = TRUE;
-  for (p = 1; p <= mvSize; p++) {
+  for (p = 1; p <= mvSignSize; p++) {
     for (i = 1; i <= mRecordSize; i++) {
       if (mvIndex[p] < 0) {
         unsignedIndex = (uint) abs(mvIndex[p]);
@@ -788,7 +864,7 @@ void imputeCommon(uint      mode,
   char outcomeFlag;
   uint     mRecordSize;
   uint    *mRecordIndex;
-  uint     mvSize;
+  uint     mvSignSize;
   int    **mvSign;
   int     *mvIndex;
   double **outResponse;
@@ -811,7 +887,7 @@ void imputeCommon(uint      mode,
   rspSize = 0;  
   mvIndex = 0;  
   mvSign  = 0;  
-  mvSize  = 0;  
+  mvSignSize  = 0;  
   mRecordIndex = 0;  
   mRecordSize  = 0;  
   if ((selectionFlag != TRUE) && (selectionFlag != FALSE) && (selectionFlag != ACTIVE)) {
@@ -826,7 +902,7 @@ void imputeCommon(uint      mode,
     if (RF_fmRecordSize > 0) {
       mRecordSize = RF_fmRecordSize;
       mRecordIndex = RF_fmRecordIndex;
-      mvSize = RF_fmvSize;
+      mvSignSize = RF_fmvSignSize;
       mvSign = RF_fmvSign;
       mvIndex = RF_fmvIndex;
       maxDistributionSize = ((RF_observationSize) > (RF_forestSize)) ? (RF_observationSize) : (RF_forestSize);
@@ -840,7 +916,7 @@ void imputeCommon(uint      mode,
     if (RF_mRecordSize > 0) {
       mRecordSize = RF_mRecordSize;
       mRecordIndex = RF_mRecordIndex;
-      mvSize = RF_mvSize;
+      mvSignSize = RF_mvSignSize;
       mvSign = RF_mvSign;
       mvIndex = RF_mvIndex;
       maxDistributionSize = ((RF_observationSize) > (RF_forestSize)) ? (RF_observationSize) : (RF_forestSize);
@@ -877,14 +953,14 @@ void imputeCommon(uint      mode,
   }
   imputedValue = 0.0;  
   double *localDistribution = dvector(1, maxDistributionSize);
-  char  *naiveMvFlag = cvector(1, mvSize);
-  char **naiveSign = cmatrix(1, mRecordSize, 1, mvSize);
-  for (p = 1; p <= mvSize; p++) {
+  char  *naiveMvFlag = cvector(1, mvSignSize);
+  char **naiveSign = cmatrix(1, mRecordSize, 1, mvSignSize);
+  for (p = 1; p <= mvSignSize; p++) {
     naiveMvFlag[p] = FALSE;
   }
   for (i = 1; i <= mRecordSize; i++) {
     outcomeFlag = TRUE;
-    for (p = 1; p <= mvSize; p++) {
+    for (p = 1; p <= mvSignSize; p++) {
       naiveSign[i][p] = FALSE;  
       if (mvIndex[p] < 0) {
         unsignedIndex = (uint) abs(mvIndex[p]);
@@ -899,7 +975,7 @@ void imputeCommon(uint      mode,
         if (mvSign[unsignedIndex][i] == 1) {
           localDistributionSize = 0;
           for (tree = 1; tree <= localSerialCount; tree++) {
-            if (RF_leafCount[serialPtr[tree]] > 0) {
+            if (RF_tLeafCount[serialPtr[tree]] > 0) {
               if ((RF_dmRecordBootFlag[serialPtr[tree]][i] == selectionFlag) || (selectionFlag == ACTIVE)) {
                 if (!ISNA(RF_dmvImputation[serialPtr[tree]][i][p])) {
                   localDistribution[++localDistributionSize] = RF_dmvImputation[serialPtr[tree]][i][p];
@@ -945,12 +1021,12 @@ void imputeCommon(uint      mode,
         }  
       }  
       else {
-        p = mvSize;
+        p = mvSignSize;
       }
     }  
   }  
   outcomeFlag = TRUE;
-  for (p = 1; p <= mvSize; p++) {
+  for (p = 1; p <= mvSignSize; p++) {
     if (mvIndex[p] < 0) {
       unsignedIndex = (uint) abs(mvIndex[p]);
       valuePtr = RF_responseIn[(uint) abs(mvIndex[p])];
@@ -999,15 +1075,15 @@ void imputeCommon(uint      mode,
       }  
     }  
     else {
-      p = mvSize;
+      p = mvSignSize;
     }
   }  
   if (serialTreeID == 0) {
     free_uivector(localSerialIndex, 1, RF_forestSize);
   }
   free_dvector(localDistribution, 1, maxDistributionSize);
-  free_cvector(naiveMvFlag, 1, mvSize);
-  free_cmatrix(naiveSign, 1, mRecordSize, 1, mvSize);
+  free_cvector(naiveMvFlag, 1, mvSignSize);
+  free_cmatrix(naiveSign, 1, mRecordSize, 1, mvSignSize);
 }
 void imputeMultipleTime (char selectionFlag) {
   double  *outTime;
@@ -1328,7 +1404,7 @@ void stackShadow (uint mode, uint treeID) {
     for (p = 1; p <= RF_rSize; p++) {
       RF_response[treeID][p] = RF_responseIn[p];
     }
-    for (p = 1; p <= RF_mvSize; p++) {
+    for (p = 1; p <= RF_mvSignSize; p++) {
       if (RF_mvIndex[p] < 0) {
         RF_response[treeID][(uint) abs(RF_mvIndex[p])] = dvector(1, RF_observationSize);
         for (i = 1; i <= RF_observationSize; i++) {
@@ -1336,7 +1412,7 @@ void stackShadow (uint mode, uint treeID) {
         }
       }
       else {
-        p = RF_mvSize;
+        p = RF_mvSignSize;
       }
     }
     if (RF_timeIndex > 0) {
@@ -1369,7 +1445,7 @@ void stackShadow (uint mode, uint treeID) {
       for (p = 1; p <= RF_xSize; p++) {
         RF_observation[treeID][p] = RF_observationIn[p];
       }
-      for (p = 1; p <= RF_mvSize; p++) {
+      for (p = 1; p <= RF_mvSignSize; p++) {
         if (RF_mvIndex[p] > 0) {
           RF_observation[treeID][(uint) RF_mvIndex[p]] = dvector(1, RF_observationSize);
           for (i = 1; i <= RF_observationSize; i++) {
@@ -1386,7 +1462,7 @@ void stackShadow (uint mode, uint treeID) {
         for (p = 1; p <= RF_frSize; p++) {
           RF_fresponse[treeID][p] = RF_fresponseIn[p];
         }
-        for (p = 1; p <= RF_fmvSize; p++) {
+        for (p = 1; p <= RF_fmvSignSize; p++) {
           if (RF_fmvIndex[p] < 0) {
             RF_fresponse[treeID][(uint) abs(RF_fmvIndex[p])] = dvector(1, RF_fobservationSize);
             for (i = 1; i <= RF_fobservationSize; i++) {
@@ -1394,7 +1470,7 @@ void stackShadow (uint mode, uint treeID) {
             }
           }
           else {
-            p = RF_fmvSize;
+            p = RF_fmvSignSize;
           }
         }
         if (RF_timeIndex > 0) {
@@ -1417,7 +1493,7 @@ void stackShadow (uint mode, uint treeID) {
         for (p = 1; p <= RF_xSize; p++) {
           RF_fobservation[treeID][p] = RF_fobservationIn[p];
         }
-        for (p = 1; p <= RF_fmvSize; p++) {
+        for (p = 1; p <= RF_fmvSignSize; p++) {
           if (RF_fmvIndex[p] > 0) {
             RF_fobservation[treeID][(uint) RF_fmvIndex[p]] = dvector(1, RF_fobservationSize);
             for (i = 1; i <= RF_fobservationSize; i++) {
@@ -1448,12 +1524,12 @@ void unstackShadow (uint mode, uint treeID) {
     vimpShadowFlag = TRUE;
   }
   if (RF_mResponseFlag == TRUE) {
-    for (p = 1; p <= RF_mvSize; p++) {
+    for (p = 1; p <= RF_mvSignSize; p++) {
       if (RF_mvIndex[p] < 0) {
         free_dvector(RF_response[treeID][(uint) abs(RF_mvIndex[p])], 1, RF_observationSize);
       }
       else {
-        p = RF_mvSize;
+        p = RF_mvSignSize;
       }
     }
     free_vvector(RF_response[treeID], 1, RF_rSize);
@@ -1468,7 +1544,7 @@ void unstackShadow (uint mode, uint treeID) {
   }
   else {
     if(RF_mPredictorFlag == TRUE) {
-      for (p = 1; p <= RF_mvSize; p++) {
+      for (p = 1; p <= RF_mvSignSize; p++) {
         if (RF_mvIndex[p] > 0) {
           free_vvector(RF_observation[treeID][(uint) RF_mvIndex[p]], 1, RF_observationSize);
         }
@@ -1479,12 +1555,12 @@ void unstackShadow (uint mode, uint treeID) {
   if (mode == RF_PRED) {
     if (RF_frSize > 0) {
       if (RF_fmResponseFlag == TRUE) {
-        for (p = 1; p <= RF_fmvSize; p++) {
+        for (p = 1; p <= RF_fmvSignSize; p++) {
           if (RF_fmvIndex[p] < 0) {
             free_vvector(RF_fresponse[treeID][(uint) abs(RF_fmvIndex[p])], 1, RF_fobservationSize);
           }
           else {
-            p = RF_fmvSize;
+            p = RF_fmvSignSize;
           }
         }
         free_vvector(RF_fresponse[treeID], 1, RF_rSize);
@@ -1495,7 +1571,7 @@ void unstackShadow (uint mode, uint treeID) {
     }
     else {
       if(RF_fmPredictorFlag == TRUE) {
-        for (p = 1; p <= RF_fmvSize; p++) {
+        for (p = 1; p <= RF_fmvSignSize; p++) {
           if (RF_fmvIndex[p] > 0) {
             free_vvector(RF_fobservation[treeID][(uint) RF_fmvIndex[p]], 1, RF_fobservationSize);
           }
@@ -1505,3 +1581,153 @@ void unstackShadow (uint mode, uint treeID) {
     }
   }
 }
+void imputeUpdateSummaryNew (uint     mode,
+                             uint     treeID) {
+  Node     *leafNodePtr;
+  Terminal *infoNodePtr;
+  int    **mvSign;
+  uint *glmdIndex;
+  uint  glmdSize;
+  uint *glmiIndex;
+  uint  glmiSize;
+  uint *glmiSizePtr;
+  uint  gdominant;
+  uint j, r, p;
+  for (j = 1; j <= RF_tLeafCount[treeID]; j++) {
+    leafNodePtr = RF_tNodeList[treeID][j];
+    infoNodePtr = RF_mTerminalInfo[treeID][j];
+    if (xferMissingness(mode, leafNodePtr, infoNodePtr)) {
+      if ((mode == RF_GROW) || (mode == RF_REST)) {
+        if ((infoNodePtr -> lmvIndexSize) > (infoNodePtr -> lmrIndexSize)) {
+          gdominant = infoNodePtr -> dominant = V_DOMINANT;
+          glmdSize  = infoNodePtr -> lmvIndexSize; 
+          glmdIndex = infoNodePtr -> lmvIndex; 
+          glmiSize  = infoNodePtr -> lmrIndexSize; 
+          glmiIndex = infoNodePtr -> lmrIndex; 
+        }
+        else {
+          gdominant = infoNodePtr -> dominant = R_DOMINANT;
+          glmdSize  = infoNodePtr -> lmrIndexSize; 
+          glmdIndex = infoNodePtr -> lmrIndex; 
+          glmiSize  = infoNodePtr -> lmvIndexSize; 
+          glmiIndex = infoNodePtr -> lmvIndex; 
+        } 
+        stackTermLMISizePtr(infoNodePtr, glmiSize);
+        glmiSizePtr = infoNodePtr -> lmiSizePtr;
+        mvSign = RF_mvSign;
+      }
+      else {
+        if ((infoNodePtr -> flmvIndex) > (infoNodePtr -> flmrIndex)) {
+          gdominant = infoNodePtr -> fdominant = V_DOMINANT;
+          glmdSize  = infoNodePtr -> flmvIndexSize; 
+          glmdIndex = infoNodePtr -> flmvIndex; 
+          glmiSize  = infoNodePtr -> flmrIndexSize; 
+          glmiIndex = infoNodePtr -> flmrIndex; 
+        }
+        else {
+          gdominant = infoNodePtr -> fdominant = R_DOMINANT;
+          glmdSize  = infoNodePtr -> flmrIndexSize;
+          glmdIndex = infoNodePtr -> flmrIndex; 
+          glmiSize  = infoNodePtr -> flmvIndexSize; 
+          glmiIndex = infoNodePtr -> flmvIndex; 
+        } 
+        stackTermFLMISizePtr(infoNodePtr, glmiSize);
+        glmiSizePtr = infoNodePtr -> flmiSizePtr;
+        mvSign = RF_fmvSign;
+      }
+      if (gdominant == V_DOMINANT) {
+        for (r = 1; r <= glmiSize; r++) {
+          glmiSizePtr[r] = 0;
+          for (p = 1; p <= glmdSize; p++) {
+            if (mvSign[glmdIndex[p]][glmiIndex[r]] == 1) {
+              glmiSizePtr[r] ++;
+            }
+          }
+        }
+      }
+      else {
+        for (p = 1; p <= glmiSize; p++) {
+          glmiSizePtr[p] = 0;
+          for (r = 1; r <= glmdSize; r++) {
+            if (mvSign[glmiIndex[p]][glmdIndex[r]] == 1) {
+              glmiSizePtr[p] ++;
+            }
+          }
+        }
+      }
+    }
+  }
+}
+char xferMissingness(uint mode, Node *source, Terminal *destination) {
+  uint p;
+  char legalResult;
+  char xferFlag;
+  legalResult = xferFlag = FALSE;  
+  if ((mode == RF_GROW) || (mode == RF_REST)) {
+    if (RF_mRecordSize > 0) {
+      if (source -> lmvIndexActualSize > 0) {
+        stackTermLMVIndex(destination, source -> lmvIndexActualSize);
+        for (p = 1; p <= source -> lmvIndexActualSize; p++) {
+          (destination -> lmvIndex)[p] = (source -> lmvIndex)[p]; 
+        } 
+      }
+      if (source -> lmrIndexActualSize > 0) {
+        stackTermLMRIndex(destination, source -> lmrIndexActualSize);
+        for (p = 1; p <= source -> lmrIndexActualSize; p++) {
+          (destination -> lmrIndex)[p] = (source -> lmrIndex)[p]; 
+        } 
+      }
+      if ((source -> lmvIndexActualSize > 0) && (source -> lmrIndexActualSize > 0)) {
+        legalResult = TRUE;
+        xferFlag = TRUE;
+      }
+      else {
+        if ((source -> lmvIndexActualSize == 0) && (source -> lmrIndexActualSize == 0)) {
+          legalResult = TRUE;
+          xferFlag = FALSE;
+        }
+        else {
+          legalResult = FALSE;
+          xferFlag = FALSE;
+        }
+      }
+    }
+  }
+  else {
+    if (RF_fmRecordSize > 0) {       
+      if (source -> flmvIndexActualSize > 0) {
+        stackTermFLMVIndex(destination, source -> flmvIndexActualSize);
+        for (p = 1; p <= source -> flmvIndexActualSize; p++) {
+          (destination -> flmvIndex)[p] = (source -> flmvIndex)[p]; 
+        } 
+      }
+      if (source -> flmrIndexActualSize > 0) {
+        stackTermFLMRIndex(destination, source -> flmrIndexActualSize);
+        for (p = 1; p <= source -> flmrIndexActualSize; p++) {
+          (destination -> flmrIndex)[p] = (source -> flmrIndex)[p]; 
+        } 
+      }
+      if ((source -> flmvIndexActualSize > 0) && (source -> flmrIndexActualSize > 0)) {
+        legalResult = TRUE;
+        xferFlag = TRUE;
+      }
+      else {
+        if ((source -> flmvIndexActualSize == 0) && (source -> flmrIndexActualSize == 0)) {
+          legalResult = TRUE;
+          xferFlag = FALSE;
+        }
+        else {
+          legalResult = FALSE;
+          xferFlag = FALSE;
+        }
+      }
+    }
+  }
+  if (legalResult == FALSE) {
+    Rprintf("\nRF-SRC:  *** ERROR *** ");
+    Rprintf("\nRF-SRC:  Attempt to transfer missing information with either no on inconsistent missingness in mode:  %10d", mode);
+    Rprintf("\nRF-SRC:  Please Contact Technical Support.");
+    error("\nRF-SRC:  The application will now exit.\n");
+  }
+  return xferFlag;
+}
