@@ -2,7 +2,7 @@
 ////**********************************************************************
 ////
 ////  RANDOM FORESTS FOR SURVIVAL, REGRESSION, AND CLASSIFICATION (RF-SRC)
-////  Version 1.1.0
+////  Version 1.2
 ////
 ////  Copyright 2012, University of Miami
 ////
@@ -52,7 +52,7 @@
 ////    5425 Nestleway Drive, Suite L1
 ////    Clemmons, NC 27012
 ////
-////    email:  ubk@kogalur.com
+////    email:  commerce@kogalur.com
 ////    URL:    http://www.kogalur.com
 ////    --------------------------------------------------------------
 ////
@@ -67,10 +67,8 @@
 Terminal *makeTerminal();
 void freeTerminal(Terminal *parent);
 Node *makeNode(unsigned int xSize);
-void freeNode(Node *parent,
-              char  dFlag
-);
-void freeTerminalNodeStructures(Node *terminalNode);
+void freeNode(Node *parent);
+void freeTerminalNodeSurvivalStructures(Node *terminalNode);
 void getNodeInfo(Node *leaf);
 void setParent(
   Node *daughter,
@@ -111,10 +109,30 @@ void stackCIF(Node *tNode, unsigned int eTypeSize, unsigned int sTimeSize);
 void unstackCIF(Node *tNode);
 void stackMortality(Node *tNode, unsigned int eTypeSize);
 void unstackMortality(Node *tNode);
-void stackMVSign(Node *node, unsigned int mvSize);
+void stackMVSign(Node *node, unsigned int mvSignSize);
 void unstackMVSign(Node *node);
-void stackFMVSign(Node *node, unsigned int fmvSize);
+void stackFMVSign(Node *node, unsigned int fmvSignSize);
 void unstackFMVSign(Node *node);
+void stackNodeLMVIndex(Node *node, unsigned int size);
+void unstackNodeLMVIndex(Node *node);
+void stackNodeFLMVIndex(Node *node, unsigned int size);
+void unstackNodeFLMVIndex(Node *node);
+void stackNodeLMRIndex(Node *node, unsigned int size);
+void unstackNodeLMRIndex(Node *node);
+void stackNodeFLMRIndex(Node *node, unsigned int size);
+void unstackNodeFLMRIndex(Node *node);
+void stackTermLMVIndex(Terminal *node, unsigned int size);
+void unstackTermLMVIndex(Terminal *node);
+void stackTermFLMVIndex(Terminal *node, unsigned int size);
+void unstackTermFLMVIndex(Terminal *node);
+void stackTermLMRIndex(Terminal *node, unsigned int size);
+void unstackTermLMRIndex(Terminal *node);
+void stackTermFLMRIndex(Terminal *node, unsigned int size);
+void unstackTermFLMRIndex(Terminal *node);
+void stackTermFLMISizePtr(Terminal *tNode, unsigned int size);
+void unstackTermFLMISizePtr(Terminal *tNode);
+void stackTermLMISizePtr(Terminal *tNode, unsigned int size);
+void unstackTermLMISizePtr(Terminal *tNode);
 void stackMultiClassProb(Node *tNode, unsigned int rfCount, unsigned int *rfSize);
 void unstackMultiClassProb(Node *tNode);
 void stackSplitDepth(Node *tNode, unsigned int depth);
