@@ -2,7 +2,7 @@
 ////**********************************************************************
 ////
 ////  RANDOM FORESTS FOR SURVIVAL, REGRESSION, AND CLASSIFICATION (RF-SRC)
-////  Version 1.1.0
+////  Version 1.2
 ////
 ////  Copyright 2012, University of Miami
 ////
@@ -52,7 +52,7 @@
 ////    5425 Nestleway Drive, Suite L1
 ////    Clemmons, NC 27012
 ////
-////    email:  ubk@kogalur.com
+////    email:  commerce@kogalur.com
 ////    URL:    http://www.kogalur.com
 ////    --------------------------------------------------------------
 ////
@@ -90,13 +90,13 @@ void acquireTree(uint mode, uint r, uint b) {
   Node *parent;
   uint bootMembrIndxIter;
   char  result;
-  Node  ***nodeMembershipPtr;
+  Node  ***gNodeMembership;
   uint     obsSize;
   uint i, j;
   obsSize    = 0;  
   responsePtr = NULL;  
   predictorPtr = NULL;  
-  nodeMembershipPtr = NULL;  
+  gNodeMembership = NULL;  
   updateFlag  = FALSE;  
   multipleImputeFlag = FALSE;
   if (mode == RF_GROW) {
@@ -107,18 +107,35 @@ void acquireTree(uint mode, uint r, uint b) {
 #ifdef SUPPORT_OPENMP
 #endif
   rootPtr = makeNode(RF_xSize);  
-  RF_nodeMembership[b] = (Node **) vvector(1, RF_observationSize);
+  RF_tNodeMembership[b] = (Node **) vvector(1, RF_observationSize);
   RF_bootMembershipIndex[b] = uivector(1, RF_observationSize);
   RF_bootMembershipFlag[b] = uivector(1, RF_observationSize);
   RF_oobMembershipFlag[b] = uivector(1, RF_observationSize);
   allMembrIndx = uivector(1, RF_observationSize);
+  if ((mode == RF_GROW) || (mode == RF_REST)) {
+    obsSize = RF_observationSize; 
+    gNodeMembership = RF_tNodeMembership;
+  }
+  else {
+    if (mode == RF_PRED) {
+      obsSize = RF_fobservationSize; 
+      gNodeMembership = RF_ftNodeMembership;
+    }
+    else {
+    }
+  }
+  if (mode == RF_PRED) {
+    RF_ftNodeMembership[b] = (Node **) vvector(1, RF_fobservationSize);
+  }
   if (mode == RF_PRED) {
-    RF_fnodeMembership[b] = (Node **) vvector(1, RF_fobservationSize);
     fallMembrIndx = uivector(1, RF_fobservationSize);
   }
   else {
     fallMembrIndx = NULL;
   }
+  if (RF_ptnCount > 0) {
+    RF_pNodeMembership[b] = (Node **) vvector(1, obsSize);
+  }
   stackShadow(mode, b);
   if (mode == RF_GROW) {
     if (RF_imputeSize > 1) {
@@ -161,12 +178,17 @@ void acquireTree(uint mode, uint r, uint b) {
   bootMembrIndxIter = 0;
   for (i=1; i <= RF_observationSize; i++) {
     allMembrIndx[i] = i;
-    RF_nodeMembership[b][i] = RF_root[b];
+    RF_tNodeMembership[b][i] = RF_root[b];
     RF_bootMembershipFlag[b][i] = FALSE;
     RF_oobMembershipFlag[b][i]  = TRUE;
   }
+  if (RF_ptnCount > 0) {
+    for (i = 1; i <= obsSize; i++) {
+      RF_pNodeMembership[b][i] = gNodeMembership[b][i]; 
+    }
+  }
   if (mode == RF_GROW) {
-    RF_leafCount[b] = 0;
+    RF_tLeafCount[b] = 0;
     result = growTree (TRUE, 
                        multipleImputeFlag, 
                        b, 
@@ -178,25 +200,33 @@ void acquireTree(uint mode, uint r, uint b) {
                        0, 
                        RF_maxDepth + b,
                        & bootMembrIndxIter);
-    RF_terminalNode[b] = (Node **) vvector(1, RF_leafCount[b] + 1);
-    if ((RF_opt & OPT_MISS) | (RF_opt & OPT_OMIS)) {
-      RF_mTerminalInfo[b] = (Terminal **) vvector(1, RF_leafCount[b] + 1);
-      for (j = 1; j <= RF_leafCount[b]; j++) {
-        RF_mTerminalInfo[b][j] = makeTerminal();
-      }
+    RF_tNodeList[b] = (Node **) vvector(1, RF_tLeafCount[b] + 1);
+    RF_nodeListIndex[b] = uivector(1, RF_tLeafCount[b] + 1);
+    for (j = 1; j <= RF_tLeafCount[b]; j++) {
+      RF_tNodeList[b][j] = getTerminalNode(b, j);
     }
-    for (j = 1; j <= RF_leafCount[b]; j++) {
-      RF_terminalNode[b][j] = getTerminalNode(b, j);
+    for (j = 1; j <= RF_tLeafCount[b]; j++) {
+      RF_nodeListIndex[b][j] = j;
+    }
+    if ((RF_opt & OPT_MISS) | (RF_opt & OPT_OMIS)) {
+      RF_mTerminalInfo[b] = (Terminal **) vvector(1, RF_tLeafCount[b] + 1);
+      for (j = 1; j <= RF_tLeafCount[b]; j++) {
+        RF_mTerminalInfo[b][j] = makeTerminal();
+        RF_mTerminalInfo[b][j] -> mate = RF_tNodeList[b][j];
+        RF_mTerminalInfo[b][j] -> leafCount = RF_tNodeList[b][j] -> leafCount;
+        RF_tNodeList[b][j] -> mate = RF_mTerminalInfo[b][j];
+      }
     }
     if (result) {
       if ((RF_imputeSize > 1) && (r > 1) && (r < RF_imputeSize) ) {
         if (RF_mRecordSize > 0) {
           repMembrIndxImputed = uivector(1, RF_observationSize);
           allMembrIndxImputed = uivector(1, RF_observationSize);
-          for (j = 1; j <= RF_leafCount[b]; j++) {
-            parent = RF_terminalNode[b][j];
+          for (j = 1; j <= RF_tLeafCount[b]; j++) {
+            parent = RF_tNodeList[b][j];
             getRawNodeSize(RF_GROW, b, parent, repMembrIndxImputed, & repMembrSizeImputed, allMembrIndxImputed,  & allMembrSizeImputed);
             imputeNode(RF_GROW,
+                       FALSE,
                        FALSE,
                        b,
                        parent,
@@ -215,7 +245,7 @@ void acquireTree(uint mode, uint r, uint b) {
     if (mode != RF_REST) {
       for (i=1; i <= RF_fobservationSize; i++) {
         fallMembrIndx[i] = i;
-        RF_fnodeMembership[b][i] = RF_root[b];
+        RF_ftNodeMembership[b][i] = RF_root[b];
       }
     }
     nodeOffset = 1;
@@ -225,9 +255,10 @@ void acquireTree(uint mode, uint r, uint b) {
       mwcpPtr += RF_mwcpCount[j];
       nodeOffset += RF_nodeCount[j];
     }
-    RF_terminalNode[b] = (Node **) vvector(1, RF_leafCount[b] + 1);
+    RF_tNodeList[b] = (Node **) vvector(1, RF_tLeafCount[b] + 1);
+    RF_nodeListIndex[b] = uivector(1, RF_tLeafCount[b] + 1);
     if ((RF_opt & OPT_MISS) | (RF_opt & OPT_OMIS)) {
-      RF_mTerminalInfo[b] = (Terminal **) vvector(1, RF_leafCount[b] + 1);
+      RF_mTerminalInfo[b] = (Terminal **) vvector(1, RF_tLeafCount[b] + 1);
     }
     restoreTree(b,
                 rootPtr,
@@ -253,6 +284,16 @@ void acquireTree(uint mode, uint r, uint b) {
                                    & bootMembrIndxIter);
   }  
   if (result) {
+    if (RF_ptnCount > 0) {
+      for (i = 1; i <= obsSize; i++) {
+        RF_pNodeMembership[b][i] = gNodeMembership[b][i]; 
+      }
+      RF_pLeafCount[b] = pruneTree(mode, b, RF_ptnCount);
+      RF_pNodeList[b] = (Node **) vvector(1, RF_pLeafCount[b] + 1);
+      i = 0;
+      getPTNodeList(RF_root[b], RF_pNodeList[b], &i);
+      free_vvector(RF_pNodeList[b], 1, RF_pLeafCount[b] + 1);
+    }
     RF_oobSize[b] = 0;
     for (i=1; i <= RF_observationSize; i++) {
       if (RF_bootMembershipFlag[b][i] == FALSE) {
@@ -309,24 +350,11 @@ void acquireTree(uint mode, uint r, uint b) {
     free_uivector(fallMembrIndx, 1, RF_fobservationSize);
   }
   if (RF_opt & OPT_MEMB) {
-    if ((mode == RF_GROW) || (mode == RF_REST)) {
-      obsSize = RF_observationSize; 
-      nodeMembershipPtr = RF_nodeMembership;
-    }
-    else {
-      if (mode == RF_PRED) {
-        obsSize = RF_fobservationSize; 
-        nodeMembershipPtr = RF_fnodeMembership;
-      }
-      else {
-        Rprintf("\nRF-SRC:  *** ERROR *** ");
-        Rprintf("\nRF-SRC:  Illegal mode for SEXP membership output:  %10d", mode);
-        Rprintf("\nRF-SRC:  Please Contact Technical Support.");
-        error("\nRF-SRC:  The application will now exit.\n");
-      }
-    }
     for (i=1; i <= obsSize; i++) {
-      RF_terminalNodeMembershipPtr[b][i] = nodeMembershipPtr[b][i] -> leafCount;
+      RF_tNodeMembershipIndexPtr[b][i] = gNodeMembership[b][i] -> leafCount;
+      if (RF_ptnCount > 0) {
+        RF_pNodeMembershipIndexPtr[b][i] = RF_pNodeMembership[b][i] -> leafCount;
+      }
     }
   }
 }
@@ -334,13 +362,13 @@ void updateProximity(uint mode, uint treeID) {
   Node ***nodeMembership;
   uint    obsSize;
   uint i, j, k;
-  if (RF_leafCount[treeID] > 0) {
+  if (RF_tLeafCount[treeID] > 0) {
     if ((mode == RF_GROW) || (mode == RF_REST)) {
-      nodeMembership = RF_nodeMembership;
+      nodeMembership = RF_tNodeMembership;
       obsSize = RF_observationSize;
     }
     else {
-      nodeMembership = RF_fnodeMembership;
+      nodeMembership = RF_ftNodeMembership;
       obsSize = RF_fobservationSize;
     }
     k = 0;
@@ -359,7 +387,7 @@ void updateSplitDepth(uint treeID, Node *rootPtr, uint maxDepth) {
   double *localSplitDepth;
   uint index;
   uint i, j, k;
-  if (RF_leafCount[treeID] > 0) {
+  if (RF_tLeafCount[treeID] > 0) {
     index = 0;  
     if (RF_opt & (OPT_SPLDPTH_F | OPT_SPLDPTH_T)) {
       if (RF_opt & OPT_SPLDPTH_F) {
@@ -380,7 +408,7 @@ void updateSplitDepth(uint treeID, Node *rootPtr, uint maxDepth) {
       for (j = 1; j <= RF_xSize; j++) {
         localSplitDepth[j] = NA_REAL;
       }
-      parent = RF_nodeMembership[treeID][i];
+      parent = RF_tNodeMembership[treeID][i];
       for (k = 1; k <= parent -> depth; k++) {
         if (ISNA(localSplitDepth[(parent -> splitDepth)[k]])) {
           localSplitDepth[(parent -> splitDepth)[k]] = (double) k;
@@ -399,3 +427,109 @@ void updateSplitDepth(uint treeID, Node *rootPtr, uint maxDepth) {
     freeSplitDepth(treeID);
   }
 }
+char pruneBranch(uint mode, uint treeID, Node **nodesAtDepth, uint nadCount, uint ptnTarget, uint ptnCurrent) {
+  char pruneFlag;
+  uint obsSize;
+  uint i, j;
+  pruneFlag = TRUE;
+  double *varianceAtDepth =  dvector(1, nadCount);
+  uint   *vadSortedIndex  = uivector(1, nadCount);
+  if ((mode == RF_GROW) || (mode == RF_REST)) {
+    obsSize = RF_observationSize; 
+  }
+  else {
+      obsSize = RF_fobservationSize; 
+  }
+  for (i = 1; i <= nadCount; i++) {
+    varianceAtDepth[i] = nodesAtDepth[i] -> variance;
+  }
+  indexx(nadCount, varianceAtDepth, vadSortedIndex);
+  j = nadCount;
+  while ((j >= 1) && pruneFlag) {
+    nodesAtDepth[vadSortedIndex[j]] -> pseudoTerminal = TRUE;
+    (nodesAtDepth[vadSortedIndex[j]] -> left)  -> pseudoTerminal = FALSE;
+    (nodesAtDepth[vadSortedIndex[j]] -> right) -> pseudoTerminal = FALSE;
+    for (i = 1; i <= obsSize; i++) {
+      if ( (RF_pNodeMembership[treeID][i] == nodesAtDepth[vadSortedIndex[j]] -> left) ||
+           (RF_pNodeMembership[treeID][i] == nodesAtDepth[vadSortedIndex[j]] -> right)) {
+        RF_pNodeMembership[treeID][i] = nodesAtDepth[vadSortedIndex[j]];
+      }
+    }
+    j --;
+    ptnCurrent --;
+    if (ptnCurrent <= ptnTarget) {
+      pruneFlag = FALSE;
+    }
+  }
+  free_dvector(varianceAtDepth, 1, nadCount);
+  free_uivector(vadSortedIndex, 1, nadCount);
+  return pruneFlag;
+}
+uint pruneTree(uint mode, uint treeID, uint ptnTarget) {
+  Node **nodesAtDepth;
+  uint   ptnCurrent;
+  uint   nadCount;
+  uint   tagDepth;
+  char   pruneFlag;
+  uint   i;
+  if (ptnTarget < 1) {
+    Rprintf("\nRF-SRC:  *** ERROR *** ");
+    Rprintf("\nRF-SRC:  Illegal target PTN count in pruneTree():  %10d", ptnTarget);
+    Rprintf("\nRF-SRC:  Please Contact Technical Support.");
+    error("\nRF-SRC:  The application will now exit.\n");
+  }
+  if (RF_tLeafCount[treeID] == 0) {
+    Rprintf("\nRF-SRC:  *** ERROR *** ");
+    Rprintf("\nRF-SRC:  Illegal call to pruneTree() on a rejected tree:  %10d", treeID);
+    Rprintf("\nRF-SRC:  Please Contact Technical Support.");
+    error("\nRF-SRC:  The application will now exit.\n");
+  }
+  nodesAtDepth = (Node **) vvector(1, RF_tLeafCount[treeID]);
+  ptnCurrent = RF_tLeafCount[treeID];
+  tagDepth = getMaximumDepth(RF_root[treeID]) - 1;
+  pruneFlag = (ptnCurrent > ptnTarget) && (tagDepth > 0);
+  while (pruneFlag) {
+    for (i = 1; i <= RF_tLeafCount[treeID]; i++) {
+      nodesAtDepth[i] = NULL;
+    }
+    nadCount = 0;
+    getNodesAtDepth(RF_root[treeID], tagDepth, nodesAtDepth, &nadCount);
+    pruneFlag = pruneBranch(mode, treeID, nodesAtDepth, nadCount, ptnTarget, ptnCurrent);
+    if(pruneFlag) {
+      ptnCurrent -= nadCount;
+      tagDepth --;
+    }
+    else {
+      ptnCurrent = ptnTarget;
+    }
+  }
+  free_vvector(nodesAtDepth, 1, RF_tLeafCount[treeID]);
+  return ptnCurrent;
+}
+void unstackAuxiliary(uint mode, uint b) {
+  uint obsSize;
+  obsSize = 0;  
+  free_vvector(RF_tNodeMembership[b], 1, RF_observationSize);
+  free_uivector(RF_bootMembershipIndex[b], 1, RF_observationSize);
+  free_uivector(RF_bootMembershipFlag[b], 1, RF_observationSize);
+  free_uivector(RF_oobMembershipFlag[b], 1, RF_observationSize);
+  free_vvector(RF_tNodeList[b], 1, RF_tLeafCount[b] + 1);
+  free_uivector(RF_nodeListIndex[b], 1, RF_tLeafCount[b] + 1);
+  if (mode == RF_PRED) {
+    free_vvector((Node **) RF_ftNodeMembership[b],  1, RF_fobservationSize);
+  }
+  if (RF_ptnCount > 0) {
+    if ((mode == RF_GROW) || (mode == RF_REST)) {
+      obsSize = RF_observationSize; 
+    }
+    else {
+      if (mode == RF_PRED) {
+        obsSize = RF_fobservationSize; 
+      }
+      else {
+      }
+    }
+    free_vvector(RF_pNodeMembership[b], 1, obsSize);
+  }
+  unstackShadow(mode, b);
+}
